import os
import re
import sys
import json
import shutil
import logging
import platform
import subprocess
import dataclasses
from pathlib import Path
from typing import List, Dict
from dataclasses import dataclass

# pipi
import requests
from rich import print as rprint
from rich.console import Console
from rich.logging import RichHandler
from rich.text import Text
from rich.table import Table

try:
    from magic.compat import detect_from_filename
except ImportError:
    rprint(
        "[red]Failed to find libmagic.  Check your installation\n"
        "refer this url to install libmagic first: https://github.com/ahupp/python-magic#installation [/]"
    )
    sys.exit(1)

# logging.basicConfig(level=logging.INFO)


# locals
from InstallRelease.constants import _colors

console = Console()


def _logger(flag: str = "", format: str = ""):
    if format == "" or format == None:
        format = "%(levelname)s|%(name)s| %(message)s"

    # message
    logger = logging.getLogger(__name__)

    if os.environ.get(flag) != None:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    # create console handler and set level to debug
    # ch = logging.StreamHandler()
    # ch.setLevel(logging.DEBUG)
    # # create formatter
    # # add formatter to ch
    # formatter = logging.Formatter(format)
    # ch.setFormatter(formatter)

    # # add ch to logger
    # logger.addHandler(ch)
    handler = RichHandler(log_time_format="")
    logger.addHandler(handler)
    return logger


# message
# export loglevel=true
logger = _logger("loglevel")


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if dataclasses.is_dataclass(o):
            return dataclasses.asdict(o)
        return super().default(o)


def FilterDataclass(data: dict, obj):
    """"""

    out: dict = dict()
    names = set([f.name for f in dataclasses.fields(obj)])
    for k, v in data.items():
        if k in names:
            out[k] = v
    return obj(**out)


def isNone(val):
    if val == None:
        return True
    elif isinstance(val, str) and val != "":
        return False
    elif isinstance(val, dict) and val != {}:
        return False
    elif isinstance(val, list) and val != []:
        return False
    return True


@dataclass
class ShellOutputs:
    stdout: List[str]
    stderr: List[str]
    returncode: int


class Shell:

    line_breaks = "\n"
    popen_args = {"shell": True, "stdout": subprocess.PIPE, "stderr": subprocess.PIPE}

    def console_to_str(self, s):
        console_encoding = sys.__stdout__.encoding

        """ From pypa/pip project, pip.backwardwardcompat. License MIT. """
        if s is None:
            return
        try:
            return s.decode(console_encoding, "ignore")
        except UnicodeDecodeError:
            return s.decode("utf_8", "ignore")

    def str_from_console(self, s):
        text_type = str

        try:
            return text_type(s)
        except UnicodeDecodeError:
            return text_type(s, encoding="utf_8")

    def cmd(self, cmd) -> ShellOutputs:
        try:
            process = subprocess.Popen(cmd, **self.popen_args)
            stdout, stderr = process.communicate()
            returncode = process.returncode

        except Exception as e:
            logger.error("Exception for %s: \n%s" % (subprocess.list2cmdline(cmd), e))

        returncode = returncode

        stdout = self.console_to_str(stdout)
        stdout = stdout.split(self.line_breaks)
        stdout = list(filter(None, stdout))  # filter empty values

        stderr = self.console_to_str(stderr)
        stderr = stderr.split(self.line_breaks)
        stderr = list(filter(None, stderr))  # filter empty values

        if "has-session" in cmd and len(stderr):
            if not stdout:
                stdout = stderr[0]
        logger.debug(f"stdout for {cmd}:\n{stdout}")

        return ShellOutputs(stdout=stdout, stderr=stderr, returncode=returncode)


def sh(command: str):
    s = Shell()
    return s.cmd(command)


def mkdir(path: str):
    file_path = Path(path)
    file_path = file_path.expanduser()

    if not file_path.is_dir():
        logger.debug(f"creating dir: {file_path.absolute()}")
        os.makedirs(name=file_path.absolute())
    else:
        ...


def download(url: str, at: str):
    """Download a file"""

    file = requests.get(url, stream=True)
    if not os.path.exists(at):
        os.makedirs(at)

    file_name: str = url.split("/")[-1]
    if file.status_code == 200:
        with open(f"{at}/{file_name}", "wb") as fw:
            fw.write(file.content)
        logger.info(f"""Downloaded: \'{file_name}\' at {at}""")
        return f"{at}/{file_name}"
    else:
        logger.info(f"url: {url}, status_code: {file.status_code}")
        exit()


def extract(path: str, at: str):
    """Extract tar file"""

    try:
        system = platform.system().lower()
        file_info = detect_from_filename(path)

        if file_info.mime_type == "application/x-7z-compressed":
            if system in ["linux"]:
                cmd = f"7z x {path} -o{at}"
                logger.debug("command: " + cmd)
                sh(cmd)
            elif system == "windows":
                # 'C:\Program Files\\7-zip\\7z.exe'
                ...
        else:
            shutil.unpack_archive(path, at)

        return True
    except:
        logger.error(f"can't extract: {path}")
        raise Exception("Invalid file")


def listItemsMatcher(patterns: List[str], word: str) -> float:
    """
    eg: listItemsMatcher(patterns=['a','b'], word='a-cc') --> 0.5
    """

    count = 0

    for pattern in patterns:
        if re.search(pattern.lower(), word.lower()):
            count += 1

    if count == 0:
        return 0

    return count / len(patterns)


def show_table(
    data: List[Dict], ignore_keys: List = [], title: str = "", border_style=""
):
    """rich table"""

    def dict_list_tbl(items=List[dict], ignore_keys: list = []):
        keys = []
        data = []

        for item in items:
            _tmp: tuple = ()
            for key in [i for i in item.keys() if i not in ignore_keys]:

                if key not in keys:
                    keys.append(key)
                _tmp += (str(item[key]),)
            data.append(_tmp)

        return keys, data

    text = Text(title, style=_colors["light_green"])

    print()

    table = Table(title=text, style=_colors["purple"], border_style=border_style)
    columns, rows = dict_list_tbl(data, ignore_keys)

    colors = {
        0: _colors["yellow"],
        1: _colors["red"],
        2: _colors["green"],
        3: _colors["cyan"],
        4: _colors["blue"],
    }

    for count, col in enumerate(columns):
        color = colors[count % len(colors)]
        table.add_column(col, justify="left", style=color, no_wrap=True)
    for row in rows:
        table.add_row(*row)

    console = Console()
    console.print(table)



import sys
import re
import glob
import platform
from typing import List

# pipi
import requests
from requests.auth import HTTPBasicAuth
from magic.compat import detect_from_filename

# locals
from InstallRelease.utils import logger, listItemsMatcher, extract, download, sh, isNone
from InstallRelease.data import (
    GithubRelease,
    GithubReleaseAssets,
    GithubRepoInfo,
    _platform_words,
)
from InstallRelease.constants import HOME

# --------------- CODE ------------------

__exec_pattern = r"application\/x-(\w+-)?(executable|binary)"


class GithubInfo:
    owner = ""
    repo_name = ""

    headers = {"Accept": "application/vnd.github.v3+json"}
    response = None

    # https://api.github.com/repos/OWNER/REPO/releases/tags/TAG
    # https://api.github.com/repos/OWNER/REPO/releases/latest

    def __init__(self, repo_url, data: dict = {}, token: str = "") -> None:
        if "https://github.com/" not in repo_url:
            logger.error("repo url must contain 'github.com'")
            sys.exit(1)

        if repo_url[-1] == "/":
            repo_url = repo_url[:-1]

        repo_url_attr: list = repo_url.split("/")

        self.repo_url: str = repo_url
        self.owner, self.repo_name = repo_url_attr[-2], repo_url_attr[-1]
        self.api = f"https://api.github.com/repos/{self.owner}/{self.repo_name}"
        self.token = token

        self.data = data
        self.info: GithubRepoInfo = GithubRepoInfo(**self._req(self.api))

    def _req(self, url):

        if not isNone(self.token):
            auth = HTTPBasicAuth("user", self.token)
        else:
            logger.debug("Token not set")
            auth = HTTPBasicAuth("user", "pass")

        response = requests.get(
            url,
            headers=self.headers,
            auth=auth,
            json=self.data,
        ).json()

        if isinstance(response, dict):
            if response.get("message"):
                logger.error(response)
                exit(1)

        return response

    def repository(self):
        return self._req(self.api)

    def release(self, tag_name: str = ""):

        if tag_name == "":
            api = self.api + "/releases"
        else:
            api = self.api + "/releases/tags/" + tag_name

        # Github release info api
        if not self.response:
            logger.debug(f"get: {api}")
            req = self._req(api)

            if not isinstance(req, list):
                req = [req]

            self.response = [
                GithubRelease(
                    url=self.repo_url,
                    assets=r["assets"],
                    tag_name=r["tag_name"],
                    prerelease=r["prerelease"],
                    published_at=r["published_at"],
                    name=self.repo_name,
                )
                for r in req
            ]

        return self.response


class installRelease:
    USER: str
    SUDO_USER: str

    bin_path = {
        "linux": {"local": f"{HOME}/.local/bin", "global": "/usr/local/bin"},
        "darwin": {"local": f"{HOME}/.local/bin", "global": "/usr/local/bin"},
    }

    def __init__(self, source: str, name: str = None) -> None:
        pl = platform.system()
        self.paths = self.bin_path[pl.lower()]
        self.pl = pl
        self.source = source
        self.name = name

    def install(self, local: bool, at: str):
        system = platform.system().lower()

        if system == "linux":
            return self._install_linux(local, at)
        elif system == "darwin":
            return self._install_darwin(local, at)

    def _install_linux(self, local: bool, at: str = None):

        if local:
            cmd = f"install {self.source} {at or self.paths['local']}"
        else:
            cmd = f"sudo install {self.source} {at or self.paths['global']}"

        if self.name:
            cmd += f"/{self.name}"

        logger.info(cmd)
        out = sh(cmd)

        if out.returncode != 0:
            logger.error(out.stderr)
            return False
        else:
            logger.info(
                f"[bold yellow]Installed: {self.name}[/]", extra={"markup": True}
            )
            return True

    def _install_darwin(self, local: bool, at: str = None):
        self._install_linux(local, at)

    def _install_windows(self, local: bool, at: str = None):
        ...


def get_release(releases: List[GithubRelease], repo_url: str, extra_words: list = []):
    selected = 0.0
    name = ""

    # temp fix: install not configured for distro based on platform
    platform_words = _platform_words + ["(.tar|.zip)"]

    logger.debug(msg=("platform_words: ", platform_words))

    if len(releases) == 0:
        logger.warning(f"No releases found for: {repo_url}")
        return False

    for release in releases:
        if len(release.assets) == 0 or release.prerelease == True:
            continue
        else:
            break

    if len(release.assets) == 0:
        logger.warning(f"No release assets found for: {repo_url}")
        return False

    for i in release.assets:
        match = listItemsMatcher(
            patterns=platform_words + extra_words, word=i.name.lower()
        )
        logger.debug(f"name: '{i.name}', chances: {match}")

        if match > 0:
            if selected == 0:
                selected = match
                name = i.name
            elif match > selected:
                selected = match
                name = i.name

    if name == "":
        logger.warn(f"No match release prefix match found for {repo_url}")
        return False

    count = 0
    for i in release.assets:
        if i.name == name:
            break
        count += 1

    item = release.assets[count]
    logger.debug(
        "Selected file: \n"
        f"File: '{item.name}', content_type: '{item.content_type}', chances: {selected}"
    )
    if selected < 0.2:
        logger.warning(
            f"Final Selected item has low probability"
            f"Object: {item.name}, content_type: {item.content_type}, chances: {selected}"
        )
    return item


def extract_release(item: GithubReleaseAssets, at):

    logger.debug(f"Download path: {at}")

    path = download(item.browser_download_url, at)
    logger.debug(f"path: {path}")

    logger.debug(f"Extracting: {path}")
    if not re.match(
        pattern=__exec_pattern, string=detect_from_filename(path).mime_type
    ):
        extract(path=path, at=at)
        logger.debug("Extracting done.")

    return True


def install_bin(src: str, dest: str, local: bool, name: str = None):

    bin_files = []

    for file in glob.iglob(f"{src}/**", recursive=True):
        f = detect_from_filename(file)
        if f.name == "directory":
            continue
        elif not re.match(pattern=__exec_pattern, string=f.mime_type):
            continue

        bin_files.append(file)

    if len(bin_files) > 1 or len(bin_files) == 0:
        logger.error(f"Expect single binary file got more or less:\n{bin_files}")
        exit(1)

    irelease = installRelease(source=bin_files[0], name=name)
    irelease.install(local, at=dest)



import logging
import os

# pipi
import typer

# locals
from InstallRelease.utils import rprint, logger
from InstallRelease.cli_interact import (
    GithubInfo,
    pull_state,
    get as _get,
    upgrade as _upgrade,
    remove,
    list_install,
    show_state,
    cache_config,
    config,
)


def see_help(arg: str = ""):
    rprint(
        "This command required arguments, use "
        f"[yellow]{arg} --help[reset]"
        " to see them"
    )
    exit(1)


# cli debug type alias
__optionDebug = typer.Option(False, "-v", help="set verbose mode.")
__optionQuite = typer.Option(False, "-q", help="set quite mode.")
__optionForce = typer.Option(False, "-F", help="set force.")


def setLogger(quite: bool = None, debug: bool = None):
    if debug:
        logger.setLevel(logging.DEBUG)
    elif quite:
        logger.setLevel(logging.ERROR)


if os.environ.get("IR_DEBUG", "").lower() == "true":
    setLogger(debug=True)

app = typer.Typer(help=f"Github Release Installer, based on your system")


@app.command()
def get(
    debug: bool = __optionDebug,
    quite: bool = __optionQuite,
    url: str = typer.Argument(None, help="[URL] of github repository "),
    tag_name: str = typer.Option("", "-t", help="get a specific tag version."),
    name: str = typer.Option(
        "",
        "-n",
        help="tool name you want, Only for releases having different tools in releases",
    ),
    approve: bool = typer.Option(False, "-y", help="Approve without Prompt"),
):
    """
    | Install github release, cli tool
    """

    setLogger(quite, debug)
    if url is None or url == "":
        see_help("get")

    _get(
        GithubInfo(url, token=config.token),
        tag_name=tag_name,
        prompt=not approve,
        name=name,
    )


@app.command()
def upgrade(
    debug: bool = __optionDebug,
    quite: bool = __optionQuite,
    force: bool = __optionForce,
):
    """
    | Upgrade all installed release, cli tools
    """
    setLogger(quite, debug)
    _upgrade(force=force)


@app.command()
def ls():
    """
    | list all installed release, cli tools
    """
    list_install()


@app.command()
def rm(
    name: str = typer.Argument(None, help="name of installed tool to remove"),
    debug: bool = __optionDebug,
):
    """
    | remove any installed release, cli tools
    """
    setLogger(debug=debug)

    remove(name)


@app.command(name="config")
def Config(
    debug: bool = __optionDebug,
    token: str = typer.Option(
        "",
        "--token",
        help="set your github token to solve github api rate-limiting issue",
    ),
    path: str = typer.Option(
        "",
        "--path",
        help="set install path",
    ),
):
    """
    | Set configs for tool
    """

    setLogger(debug=debug)

    logger.info(f"Loading config: {cache_config.state_file}")

    if token != "":
        config.token = token
        logger.info("Updated token")
    if path != "":
        config.path = path
        logger.info("Updated path")

    cache_config.save()
    logger.info("Done.")


@app.command()
def state(debug: bool = __optionDebug):
    """
    | show currnet stored state
    """
    setLogger(debug=debug)
    show_state()


@app.command()
def pull(
    debug: bool = __optionDebug,
    url: str = typer.Option(
        "",
        "--url",
        help="install tools from remote state",
    ),
    override: bool = typer.Option(
        False,
        "-O",
        help="Enable Override local tool version with remote state version.",
    ),
):
    """
    | Install tools from remote state
    """
    setLogger(debug=debug)

    if url is None or url == "":
        see_help("pull")

    pull_state(url, override)


@app.command()
def version():
    """
    | print version of install-release
    """
    import InstallRelease

    print(InstallRelease.__version__)


if __name__ == "__main__":
    app()



import os
import json
import platform
from typing import Dict
from dataclasses import is_dataclass

# locals
from InstallRelease.utils import logger, EnhancedJSONEncoder, FilterDataclass, isNone


def platform_path(paths: dict, alt: str = ""):
    """provide path base on platform"""

    system = platform.system().lower()

    if not isNone(alt) and alt != "null":
        return alt

    elif paths.get(system):
        p = paths.get(system)

        if not os.path.exists(os.path.dirname(p)):
            os.makedirs(os.path.dirname(p))
        return p
    else:
        logger.error(f"No state dir path set for {system}")
        exit(1)


class State:
    def __init__(self, file_path: str, obj):
        self.state: dict = {}
        self.cache: object
        self.state_file = file_path
        self.obj = obj
        self.load()

    def load(self):
        if os.path.exists(self.state_file):
            with open(self.state_file, "r") as f:
                _s = json.load(f)
                if len(_s) == 0:
                    return
                for k in _s:
                    if is_dataclass(self.obj):
                        self.state[k] = FilterDataclass(_s[k], obj=self.obj)

    def save(self):
        with open(self.state_file, "w") as f:
            json.dump(self.state, f, indent=4, cls=EnhancedJSONEncoder)

    def get(self, key: str) -> Dict:
        return self.state.get(key)

    def set(self, key: str, value):
        self.state[key] = value

    def __getitem__(self, key: str) -> Dict:
        return self.state[key]

    def __setitem__(self, key: str, value: Dict):
        self.state[key] = value
        self.save()

    def __delitem__(self, key: str):
        del self.state[key]
        self.save()

    def __contains__(self, key: str) -> bool:
        return key in self.state

    def __len__(self) -> int:
        return len(self.state)

    def __repr__(self) -> str:
        return repr(self.state)

    def __eq__(self, other: object) -> bool:
        return self.state == other

    def __ne__(self, other: object) -> bool:
        return self.state != other



import os
from typing import Dict
from tempfile import TemporaryDirectory

# pipi
import requests
from rich.progress import track
from rich.console import Console

# locals
from InstallRelease.state import State, platform_path
from InstallRelease.data import GithubRelease, ToolConfig, irKey

from InstallRelease.data import TypeState

from InstallRelease.constants import state_path, bin_path, config_path
from InstallRelease.utils import mkdir, rprint, logger, show_table, isNone
from InstallRelease.core import get_release, extract_release, install_bin, GithubInfo


console = Console(width=40)

if os.environ.get("installState", "") == "test":
    temp_dir = "../temp"
    __spath = {
        "state_path": f"{temp_dir}/temp-state.json",
        "config_path": f"{temp_dir}/temp-config.json",
    }
    logger.info(f"installState={os.environ.get('installState')}")
else:
    __spath = {"state_path": "", "config_path": ""}

cache = State(
    file_path=platform_path(paths=state_path, alt=__spath["state_path"]),
    obj=GithubRelease,
)

cache_config = State(
    file_path=platform_path(paths=config_path, alt=__spath["config_path"]),
    obj=ToolConfig,
)


def load_config():
    config: ToolConfig = cache_config.state.get("config")

    if config != None:
        return config
    else:
        cache_config.set("config", ToolConfig())
        cache_config.save()
        return ToolConfig()


config: ToolConfig = load_config()

dest = platform_path(paths=bin_path, alt=config.path)

# ------- cli ----------


def state_info():
    logger.debug(cache.state_file)
    logger.debug(cache_config.state_file)
    logger.debug(dest)


def get(
    repo: GithubInfo,
    tag_name: str = "",
    local: bool = True,
    prompt: bool = False,
    name: str = None,
):
    state_info()

    releases = repo.release(tag_name=tag_name)

    if not len(releases) > 0:
        logger.error(f"No releases found: {repo.repo_url}")
        return

    if isNone(name):
        toolname = releases[0].name
    else:
        toolname = name

    at = TemporaryDirectory(prefix=f"dn_{repo.repo_name}_")

    _gr = get_release(releases=releases, repo_url=repo.repo_url, extra_words=[toolname])

    logger.debug(_gr)

    if _gr == False:
        return
    else:
        if prompt != False:

            rprint(
                f"\n[green bold]📑 Repo     : {repo.info.full_name}"
                f"\n[blue]🌟 Stars    : {repo.info.stargazers_count}"
                f"\n[magenta]🔮 Language : {repo.info.language}"
                f"\n[yellow]🔥 Title    : {repo.info.description}"
            )
            show_table(
                data=[
                    {
                        "Name": toolname,
                        "Selected Item": _gr.name,
                        "Version": releases[0].tag_name,
                        "Size Mb": _gr.size_mb(),
                        "Downloads": _gr.download_count,
                    }
                ],
                title=f"🚀 Install: {toolname}",
            )
            rprint(f"[color(6)]\nPath: {dest}")
            rprint("[color(34)]Install this tool (Y/n): ", end="")
            yn = input()
            if yn.lower() != "y":
                return

        extract_release(item=_gr, at=at.name)

    releases[0].assets = [_gr]

    mkdir(dest)
    install_bin(src=at.name, dest=dest, local=local, name=toolname)
    cache.set(f"{repo.repo_url}#{toolname}", value=releases[0])
    cache.save()


def upgrade(force: bool = False):

    state_info()

    state: TypeState = cache.state

    upgrades: Dict[str, GithubInfo] = {}
    for k in track(state, description="Fetching..."):
        i = irKey(k)

        repo = GithubInfo(i.url, token=config.token)
        rprint(f"Fetching: {k}")
        releases = repo.release()

        if releases[0].published_dt() > state[k].published_dt() or force == True:
            upgrades[i.name] = repo

    # ask prompt to upgrade listed tools
    if len(upgrades) > 0:

        rprint("\n[bold magenta]Following tool will get upgraded.\n")
        console.print("[bold yellow]" + " ".join(upgrades.keys()))
        rprint("[bold blue]Upgrade these tools, (Y/n):", end=" ")

        r = input()
        if r.lower() != "y":
            return
    else:
        rprint("[bold green]All tools are onto latest version")
        return

    for name in track(upgrades, description="Upgrading..."):
        repo = upgrades[name]
        releases = repo.release()
        k = f"{repo.repo_url}#{name}"

        rprint(
            "[bold yellow]"
            f"Updating: {name}, {state[k].tag_name} => {releases[0].tag_name}"
            "[/]"
        )
        get(repo, prompt=False, name=name)


def show_state():

    state_info()
    if os.path.exists(cache.state_file) and os.path.isfile(cache.state_file):
        with open(cache.state_file) as f:
            print(f.read())


def list_install(state: TypeState = None, title: str = "Installed tools"):

    if state == None:
        state_info()
        state = cache.state

    _table = []
    for key in state:
        i = irKey(key)
        _table.append(
            {
                "Name": i.name,
                "Version": state[key].tag_name,
                "Url": state[key].url,
            }
        )

    show_table(_table, title=title)


def remove(name: str):

    state_info()
    state: TypeState = cache.state
    popKey = ""

    for key in state:
        i = irKey(key)
        if i.name == name:
            popKey = key
            if os.path.exists(f"{dest}/{name}"):
                os.remove(f"{dest}/{name}")
            break

    if popKey != "":
        del state[popKey]
        cache.save()
        logger.info(f"Removed: {name}")


def pull_state(url: str = "", override: bool = False):
    """
    | Install tools from remote state
    """
    logger.debug(url)

    if isNone(url):
        return

    r: dict = requests.get(url=url).json()

    data: dict = {k: GithubRelease(**r[k]) for k in r}
    state: TypeState = cache.state

    temp: Dict[str, GithubRelease] = {}

    for key in data:
        try:
            i = irKey(key)
        except:
            logger.warning(f"Invalid input: {key}")
            continue

        if state.get(key) != None:
            if state[key].tag_name == data[key].tag_name or override == False:
                logger.debug(f"Skipping: {key}")
                continue
            else:
                temp[key] = data[key]
        else:
            temp[key] = data[key]

    logger.debug(temp)

    if len(temp) == 0:
        return

    list_install(state=temp, title="Tools to be installed")
    rprint("\n[bold magenta]Following tool will get Installed.\n")
    rprint("[bold blue]Install these tools, (Y/n):", end=" ")

    _i = input()
    if _i.lower() != "y":
        return

    for key in temp:
        try:
            i = irKey(key)
        except:
            logger.warning(f"Invalid input: {key}")
            continue
        get(
            GithubInfo(i.url, token=config.token),
            tag_name=temp[key].tag_name,
            prompt=False,
            name=i.name,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['InstallRelease']

package_data = \
{'': ['*']}

install_requires = \
['python-magic>=0.4.27,<0.5.0', 'requests', 'rich', 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['install-release = InstallRelease.cli:app']}

setup_kwargs = {
    'name': 'install-release',
    'version': '0.2.6',
    'description': 'A cli tool to install tools based on your device info directly from github releases and keep them updated.',
    'long_description': '# install-release 🚀\n[![Python Version](https://img.shields.io/badge/Python-3.8_to_3.10-xx.svg)](https://shields.io/) [![Downloads](https://static.pepy.tech/personalized-badge/install-release?&units=international_system&left_color=black&right_color=orange&left_text=Downloads)](https://pepy.tech/project/install-release)\n\n`install-release` is a cli tool to install any tool for your device directly from their github releases and keep them updated.\n\nThis can be any tool you want to install, which is pre-compiled for your device and present on github releases.\n\n> Also it\'s mainly for installing tools that are not directly available in the officially by package managers `apt, yum, pacman` etc.\n\n```bash\n# Install install-release\npip install -U install-release\n```\n\n```\n# Example Installation a tool named `gron` directly from its github releases\n\n# install-release get [GITHUB-URL]\n\n❯ install-release get https://github.com/tomnomnom/gron \n```\n\n![demo](.github/images/demo.png)\n\n\nChecking for gron is installed using installed-release:\n\n```\n❯ which gron\n/home/noobi/.release-bin/gron\n\n❯ gron --help\nTransform JSON (from a file, URL, or stdin) into discrete assignments to make it greppable\n... # more\n```\n\n## Prerequisites\n\n- python3.8 or higher\n\n- [libmagic](https://github.com/ahupp/python-magic#installation)\n- Default releases Installation Path is: `~/.release-bin/`,\nThis is the path where installed tools will get stored.\n\n- In order to run installed tools, you need to add the following line your `~/.bashrc` or `~/.zshrc` file:\n\n```bash\nexport PATH=$HOME/.release-bin:$PATH\n```\n\n\n## Install this package\n\n```bash\npip install -U install-release\n```\n\n\n### Example usage `install-release`\n\n\n```\n# Help page\n\n❯ install-release --help\nUsage: install-release [OPTIONS] COMMAND [ARGS]...\n\n  Github Release Installer, based on your system\n\n  Commands:\n    get      | Install github release, cli tool\n    ls       | list all installed release, cli tools\n    rm       | remove any installed release, cli tools\n    upgrade  | Upgrade all installed release, cli tools\n    state    | show currnet stored state\n    config   | Set configs for tool\n    pull     | Install tools from remote state\n    version  | print version of install-release \n```\n\nFor sub command help use: `install-release <sub-command> --help`\n\nExample: `install-release get --help`\n\n\nYou can shorten the command by setting alias to your `.bashrc` or `.zshrc`\n\n```bash\nalias ir="install-release"\n```\nafter this you can you alias directly for easiness\n\nExample: `ir get --help`\n\n#### Install tool from github releases\n\n```bash\n❯ install-release get "https://github.com/ahmetb/kubectx"\n\n📑 Repo     : ahmetb/kubectx\n🌟 Stars    : 13295\n✨ Language : Go\n🔥 Title    : Faster way to switch between clusters and namespaces in kubectl\n\n                              🚀 Install: kubectx                               \n┏━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┳━━━━━━━━━┳━━━━━━━━━┳━━━━━━━━━━━┓\n┃ Name    ┃ Selected Item                      ┃ Version ┃ Size Mb ┃ Downloads ┃\n┡━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━╇━━━━━━━━━╇━━━━━━━━━╇━━━━━━━━━━━┩\n│ kubectx │ kubectx_v0.9.4_linux_x86_64.tar.gz │ v0.9.4  │ 1.0     │ 43811     │\n└─────────┴────────────────────────────────────┴─────────┴─────────┴───────────┘\nInstall this tool (Y/n): y\n INFO     Downloaded: \'kubectx_v0.9.4_linux_x86_64.tar.gz\' at /tmp/dn_kubectx_ph6i7dmk                                                               utils.py:159\n INFO     install /tmp/dn_kubectx_ph6i7dmk/kubectx /home/noobi/.release-bin/kubectx                                                                  core.py:132\n INFO     Installed: kubectx\n```\n```\n# checking if kubectx is installed\n❯ which kubectx\n/home/noobi/.release-bin/kubectx\n\n❯ kubectx --version\n0.9.4\n```\n\n#### List installed tools\n\n```bash\n❯ install-release ls\n\n                       Installed tools                        \n┏━━━━━━━━━━━┳━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓\n┃ Name      ┃ Version ┃ Url                                  ┃\n┡━━━━━━━━━━━╇━━━━━━━━━╇━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┩\n│ terrascan │ v1.15.2 │ https://github.com/tenable/terrascan │\n│ gron      │ v0.7.1  │ https://github.com/tomnomnom/gron    │\n│ kubectx   │ v0.9.4  │ https://github.com/ahmetb/kubectx    │\n└───────────┴─────────┴──────────────────────────────────────┘    \n```\n\n#### Remove installed release\n\n```bash\n# Remove installed release\n\n❯ install-release rm gron\n    \nINFO     Removed: gron           \n```\n\n#### Update all previously installed tools to the latest version\n\n```bash\n❯ install-release upgrade\n\nFetching: https://github.com/tenable/terrascan#terrascan\nFetching: https://github.com/ahmetb/kubectx#kubectx\n\nFollowing tools will be upgraded:\n\nterrascan\n\nUpgrade these tools, (Y/n): y\n\nUpdating: terrascan, v1.15.0 => v1.15.2\n INFO     Downloaded: \'terrascan_1.15.2_Linux_x86_64.tar.gz\' at /tmp/dn_terrascan_0as71a6v\n INFO     install /tmp/dn_terrascan_0as71a6v/terrascan ~/.release-bin/terrascan\n INFO     Installed: terrascan\n\nProgress... ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 100% 0:00:00 \n```\n\n#### Config tool installation path\n\n```bash\n❯ install-release config --path ~/.local/bin\n\nINFO   updated path to:  ~/.local/bin\nINFO   Done\n```\n\n#### Config github to token for higher rate limit\n\n\n\n```bash\n❯ install-release config --token [your github token]\n\nINFO: Update token\nINFO: Done.\n```\n',
    'author': 'Rishang',
    'author_email': 'rishangbhavsarcs@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Rishang/install-releases',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os
import platform


HOME = os.path.expanduser("~")

__bin_at__ = ".release-bin"
__dir_name__ = "install_release"

__state_at__ = f"{__dir_name__}/state.json"
__config_at__ = f"{__dir_name__}/config.json"

_colors = {
    "green": "#8CC265",
    "light_green": "#D0FF5E bold",
    "blue": "#4AA5F0",
    "cyan": "#76F6FF",
    "yellow": "#F0A45D bold",
    "red": "#E8678A",
    "purple": "#8782E9 bold",
}


state_path = {
    "linux": f"{HOME}/.config/{__state_at__}",
    "darwin": f"{HOME}/Library/.config/{__state_at__}",
}

config_path = {
    "linux": f"{HOME}/.config/{__config_at__}",
    "darwin": f"{HOME}/Library/.config/{__config_at__}",
}

bin_path = {
    "linux": f"{HOME}/{__bin_at__}",
    "darwin": f"{HOME}/{__bin_at__}",
}


def platform_words() -> list:

    aliases = {
        "x86_64": ["x86", "x64", "amd64", "amd", "x86_64"],
        "aarch64": ["arm64", "aarch64"],
    }

    platform_words = []

    platform_words += [platform.system().lower(), platform.architecture()[0]]

    for alias in aliases:
        if platform.machine().lower() in aliases[alias]:
            platform_words += aliases[alias]

    try:
        sys_alias = platform.platform().split("-")[0].lower()

        if platform.system().lower() != sys_alias:
            platform_words.append(sys_alias)
    except:
        ...

    return platform_words



from datetime import datetime
from typing import List, Dict, Optional
from dataclasses import dataclass, fields, field

# locals
from InstallRelease.constants import platform_words

_platform_words = platform_words()


exception_compressed_mime_type = [
    "application/x-7z-compressed",
]


@dataclass
class OsInfo:
    architecture: List[str]
    platform: str
    platform_words: List[str]


@dataclass
class GithubRepoInfo:
    name: str
    full_name: str
    html_url: str
    description: str
    language: str
    stargazers_count: int

    def __init__(self, **kwargs):
        names = set([f.name for f in fields(self)])
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)


@dataclass
class GithubReleaseAssets:
    browser_download_url: str
    content_type: str
    created_at: str
    download_count: int
    id: int
    name: str
    node_id: str
    size: int
    state: str
    updated_at: str

    def __init__(self, **kwargs):
        names = set([f.name for f in fields(self)])
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

    def __post_init__(self):
        self.updated_at_dt = datetime.strptime(self.created_at, "%Y-%m-%dT%XZ")

    def size_mb(self) -> float:
        return float(str(self.size / 1000000)[:4])


@dataclass
class GithubRelease:
    url: str
    name: str
    tag_name: str
    # author: dict
    # target_commitish: str
    # draft: bool
    prerelease: bool
    published_at: str
    assets: List[GithubReleaseAssets]
    # install_path: Optional[str] = field(default_factory=str)

    def __post_init__(self):
        self.assets = [GithubReleaseAssets(**a) for a in self.assets]

    def published_dt(self):
        return datetime.strptime(self.published_at, "%Y-%m-%dT%H:%M:%SZ")


@dataclass
class ToolConfig:
    token: Optional[str] = field(default_factory=str)
    path: Optional[str] = field(default_factory=str)


class irKey:
    def __init__(self, value):
        self.name = value.split("#")[-1]
        self.url = value.split("#")[0]


# ---------- Type Aliases ----------- #

TypeState = Dict[str, GithubRelease]
